import React, { ReactElement } from "react";
import type { CompiledNoCodeComponentProps } from "../../../types";
import type { RichTextBlockElementComponentConfig } from "./$richTextBlockElement";

type RichTextBlockElementProps = CompiledNoCodeComponentProps<
  RichTextBlockElementComponentConfig["_component"],
  Pick<RichTextBlockElementComponentConfig, "type">
> & {
  elements: Array<ReactElement>;
  Paragraph: ReactElement;
  BulletedList: ReactElement;
  NumberedList: ReactElement;
};

export function RichTextBlockElementClient(props: RichTextBlockElementProps) {
  const {
    type,
    BulletedList,
    elements: Elements,
    NumberedList,
    Paragraph,
  } = props;

  const elements = Elements.map((Element, index) => (
    <Element.type {...Element.props} key={index} />
  ));

  if (type === "paragraph") {
    return <Paragraph.type {...Paragraph.props}>{elements}</Paragraph.type>;
  }

  if (type === "bulleted-list") {
    return (
      <BulletedList.type {...BulletedList.props}>{elements}</BulletedList.type>
    );
  }

  if (type === "numbered-list") {
    return (
      <NumberedList.type {...NumberedList.props}>{elements}</NumberedList.type>
    );
  }

  if (process.env.NODE_ENV === "development") {
    console.warn(`Unknown @easyblocks/rich-text-block-element type "${type}"`);
  }

  return <div>{elements}</div>;
}
